class iTachException(Exception):
    def __init__(self, response):
        if not isinstance(response, str):
            pass
        elif response.startswith("ERR_01"):
            message = "Invalid command. Command not found."
        elif response.startswith("ERR_02"):
            message = "Invalid module address (does not exist)."
        elif response.startswith("ERR_03"):
            message = "Invalid connector address (does not exist)."
        elif response.startswith("ERR_04"):
            message = "Invalid ID value."
        elif response.startswith("ERR_05"):
            message = "Invalid frequency value"
        elif response.startswith("ERR_06"):
            message = "Invalid repeat value."
        elif response.startswith("ERR_07"):
            message = "Invalid offset value."
        elif response.startswith("ERR_08"):
            message = "Invalid pulse count."
        elif response.startswith("ERR_09"):
            message = "Invalid pulse data."
        elif response.startswith("ERR_10"):
            message = "Uneven amount of <on|off> statements."
        elif response.startswith("ERR_11"):
            message = "No carriage return found."
        elif response.startswith("ERR_12"):
            message = "Repeat count exceeded."
        elif response.startswith("ERR_13"):
            message = "IR command sent to input connector."
        elif response.startswith("ERR_14"):
            message = "Blaster command sent to non-blaster connector."
        elif response.startswith("ERR_15"):
            message = "No carriage return before buffer full."
        elif response.startswith("ERR_16"):
            message = "No carriage return."
        elif response.startswith("ERR_17"):
            message = "Bad command syntax."
        elif response.startswith("ERR_18"):
            message = "Sensor command sent to non-input connector."
        elif response.startswith("ERR_19"):
            message = "Repeated IR transmission failure."
        elif response.startswith("ERR_20"):
            message = "Above designated IR <on|off> pair limit."
        elif response.startswith("ERR_21"):
            message = "Symbol odd boundary."
        elif response.startswith("ERR_22"):
            message = "Undefined symbol."
        elif response.startswith("ERR_23"):
            message = "Unknown option."
        elif response.startswith("ERR_24"):
            message = "Invalid baud rate setting."
        elif response.startswith("ERR_25"):
            message = "Invalid flow control setting."
        elif response.startswith("ERR_26"):
            message = "Invalid parity setting."
        elif response.startswith("ERR_27"):
            message = "Settings are locked."
        message = response



"""device"""
import socket
from contextlib import closing

from .exception import iTachException


class iTach(object):
    """iTach class"""
    devices = {}

    def __init__(self, ipaddress="192.168.1.111", port=4998):
        """init method"""
        self.ipaddress = ipaddress
        self.port = port

    def __repr__(self):
        return "iTach(devices=%s, ipaddress=%s, port=%d)" % (
            self.devices, self.ipaddress, self.port)

    def _check_ip(self):
        """checks if ipaddress valid"""
        with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
            sock.settimeout(3)
            return sock.connect_ex((self.ipaddress, self.port)) == 0

    def add(self, device):
        """adds device to devices"""
        device.ipaddress = self.ipaddress
        device.port = self.port
        self.devices[device.name] = device
        return device

    def send_command(self, device_name, command_name):
        """sends command to device"""
        device = self.devices[device_name]
        return device.send_command(command_name)


class VirtualDevice(object):
    """VirtualDevice class"""
    ipaddress = ""
    port = 4998

    def __init__(self, name="", commands={}):
        """init method"""
        self.name = name
        self.commands = commands
        self.generate_methods()

    def __repr__(self):
        """repr method"""
        return "VirtualDevice(name=%s, commands=%s)" % (self.name, self.commands)

    def generate_methods(self):
        for command_name in self.commands.keys():
            def fn():
                return self.send_command(command_name)
            setattr(self, command_name, fn)
            fn.__name__ = command_name

    def format_message(self, msg):
        if isinstance(msg, bytes):
            return msg.decode()
        return msg

    def format_command(self, command):
        """format command for sending"""
        if not command.endswith("\r"):
            return command + "\r"
        return command

    def format_command_name(self, command_name):
        """format command in commands for sending"""
        command = self.commands[command_name]
        return self.format_command(command).encode()

    def send_command(self, command_name, byte_size=4096, timeout=3):
        """send command from commands"""
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.settimeout(timeout)
        try:
            sock.connect((self.ipaddress, self.port))
            sock.sendall(self.format_command_name(command_name))
            return self.format_message(sock.recv(byte_size))
        except socket.error as error:
            return iTachException(error)
        finally:
            sock.close()

    def send_commands(self, command_name, repeats, byte_size=4096, timeout=3):
        """send command multiple times from command from commands"""
        for x in range(repeats):
            response = self.send_command(command_name, byte_size, timeout)
        return response


if __name__ == "__main__":
    itach = iTach(ipaddress="192.168.1.111", port=4998)
    blueray = itach.add(VirtualDevice(
        name="blueray", commands={"get_net": "get_NET,0:1"}))
    print(blueray.get_net())



from codecs import open
from os import path

from setuptools import find_packages, setup

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='itachip2ir',
    version='1.3.4',
    description='A small Python module for interacting with the Global Cache iTach WF2IR or IP2IR',
    long_description=long_description,
    url='https://github.com/thehappydinoa/itachip2ir',
    author='Aidan Holland (thehappydinoa)',
    author_email='thehappydinoa@gmail.com',

    classifiers=[
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ],

    keywords='itach ip2ir Global Cache ir smart-home',

    packages=["itachip2ir"],

    project_urls={
        'Bug Reports': 'https://github.com/thehappydinoa/itachip2ir/issues',
        'Say Thanks!': 'http://saythanks.io/to/thehappydinoa',
        'Contribute!': 'https://github.com/thehappydinoa/itachip2ir/pulls',
        'Follow Me!': 'https://twitter.com/thehappydinoa/'
    }
)
